@@ -1,5 +1,4 @@
-
-# $Id: 10_BindingsIo.pm 18283 2019-01-16 16:58:23Z dominikkarall $
+# $Id: 10_BindingsIo.pm 18283 2019-01-16 16:58:23Z fhempy $
 
 package main;
 
@@ -18,101 +17,222 @@ use CoProcess;
 use JSON;
 use Time::HiRes qw(time);
 
-sub Log($$);
-sub Log3($$$);
+our $init_done;
+our $modules;
+
 
 my $USE_DEVIO_DECODEWS = 0;
 my $timeouts = 0;
 
-sub
-BindingsIo_Initialize($)
-{
+sub BindingsIo_Initialize {
   my ($hash) = @_;
 
   $hash->{parseParams} = 1;
 
-  $hash->{DefFn}    = 'BindingsIo_Define';
-  $hash->{UndefFn}  = 'BindingsIo_Undefine';
-  $hash->{GetFn}    = 'BindingsIo_Get';
-  $hash->{SetFn}    = 'BindingsIo_Set';
-  $hash->{AttrFn}   = 'BindingsIo_Attr';
+  $hash->{DefFn}    = \&BindingsIo_Define;
+  $hash->{UndefFn}  = \&BindingsIo_Undefine;
+  $hash->{GetFn}    = \&BindingsIo_Get;
+  $hash->{SetFn}    = \&BindingsIo_Set;
+  $hash->{AttrFn}   = \&BindingsIo_Attr;
   $hash->{AttrList} = $readingFnAttributes;
-  $hash->{NotifyFn}   = 'BindingsIo_Notify';
+  $hash->{NotifyFn}   = \&BindingsIo_Notify;
 
-  $hash->{ReadFn}   = 'BindingsIo_Read';
-  $hash->{ReadyFn}  = 'BindingsIo_Ready';
-  $hash->{WriteFn}  = 'BindingsIo_Write';
+  $hash->{ReadFn}   = \&BindingsIo_Read;
+  $hash->{ReadyFn}  = \&BindingsIo_Ready;
+  $hash->{WriteFn}  = \&BindingsIo_Write;
 
-  $hash->{Clients} = "PythonModule"; # NodeModule
+  $hash->{Clients} = "PythonModule:fhempy"; # NodeModule
 
-  return undef;
+  return ;
 }
 
-sub
-BindingsIo_Define($$$)
-{
-  my ($hash, $a, $h) = @_;
+sub BindingsIo_Define {
+  my ( $hash, $a ) = @_;
+ 
   my $name = $hash->{NAME};
+  Log3 $name, 3, q[BindingsIo v1.0.1];
 
-  Log3 $hash, 3, "BindingsIo v1.0.0";
-
-  my $bindingType = ucfirst(@$a[2]);
+  my $bindingType = $a->[3] // $a->[2];
 
   $hash->{args} = $a;
-  $hash->{argsh} = $h;
+  # $hash->{argsh} = $h; // Todo: Delete other code references
 
   my $port = 0;
   my $localServer = 1;
-  if ($bindingType eq "Python") {
-    $hash->{DeviceName} = "ws:127.0.0.1:15733";
-    $hash->{IP} = "127.0.0.1";
-    $hash->{PORT} = "15733";
-    $hash->{localBinding} = 1;
-  } else {
-    $hash->{DeviceName} = "ws:".@$a[2];
-    $hash->{IP} = substr($hash->{DeviceName}, 0, index($hash->{DeviceName}, ":"));
-    $hash->{PORT} = substr($hash->{DeviceName}, index($hash->{DeviceName}, ":")+1);
-    $bindingType = ucfirst(@$a[3]);
+  
+  # Support legacy define via Python instead of fhempy statement
+  $bindingType = ($bindingType eq q[Python]) ? q[fhempy] : $bindingType;
+
+  if ($#{$a} == 3 && $bindingType eq q[fhempy])  {
+    $hash->{DeviceName} = qq{ws:$a->[2]};
     $localServer = 0;
     $hash->{localBinding} = 0;
+  } elsif ($#{$a} == 2 && ($bindingType eq q[fhempy]) )   {
+    $hash->{DeviceName} = q[ws:localhost:15733];
+    $hash->{localBinding} = 1;
+  } elsif ($#{$a} < 2) {  
+    return q[to few parameters given for define];
+  } else {  
+    return q[unsupported parameters given for define];
   }
+
+  ( undef, $hash->{IP},$hash->{PORT} ) = split ':', $hash->{DeviceName};
+
+
+  if ( $hash->{IP} !~ m/^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/ )
+  {
+     my $ip= gethostbyname($hash->{IP});
+     if (defined $ip ) { $hash->{IP} =  inet_ntoa($ip) };
+     Log3 $name, 3, qq[found IP $hash->{IP}];
+  }
+  my @BindingsIoDevices = devspec2array(qq[i:TYPE=BindingsIo:FILTER=i:IP=$hash->{IP}:FILTER=i:PORT=$hash->{PORT}]); 
+  if ( IsDevice($BindingsIoDevices[1]) )  
+  {
+    Log3 $name, 5, qq[found hostname $BindingsIoDevices[0] defined];
+    return qq[Device $BindingsIoDevices[0] with IP=$hash->{IP} is already defined and using the same port, aborting define];
+  }
+  
+  
+  $hash->{devioLoglevel} = 0;
   $hash->{nextOpenDelay} = 10;
   $hash->{BindingType} = $bindingType;
   $hash->{ReceiverQueue} = Thread::Queue->new();
-  $hash->{frame} = Protocol::WebSocket::Frame->new;
+  # send binary data via websocket
+  $hash->{binary} = 1;
 
   if ($init_done && $localServer == 1) {
     my $foundServer = 0;
-    foreach my $fhem_dev (sort keys %main::defs) {
-      $foundServer = 1 if($main::defs{$fhem_dev}{TYPE} eq $bindingType."Binding");
+    
+    # use devspec2array to find existing fhempy server device
+    my @fhempyDevices;
+    @fhempyDevices = ::devspec2array(qq[i:TYPE=$bindingType(Binding|Server)]);
+    $foundServer = ::IsDevice($fhempyDevices[0]);
+
+    if ($foundServer == 0) 
+    {
+      CommandDefine(undef, $bindingType.'server_'.$hash->{PORT}.' '.$bindingType.qq[Server $port]);
+      InternalTimer(gettimeofday()+3, 'BindingsIo_connectDev', $hash, 0);
     }
-    if ($foundServer == 0) {
-      CommandDefine(undef, $bindingType."binding_".$hash->{PORT}." ".$bindingType."Binding ".$port);
-      InternalTimer(gettimeofday()+3, "BindingsIo_connectDev", $hash, 0);
-    }
-  }
-  if ($init_done && $localServer == 0) {
-    InternalTimer(gettimeofday()+3, "BindingsIo_connectDev", $hash, 0);
+  } elsif ($init_done && $localServer == 0)  {
+    InternalTimer(gettimeofday()+3, 'BindingsIo_connectDev', $hash, 0);
   }
 
-  # put in hidden room
-  CommandAttr(undef, "$name room fhempy");
-  #CommandAttr(undef, "$name verbose 5");
+  if ($init_done == 0 && $localServer == 1) {
+    readingsSingleUpdate($hash, 'state', 'Installing fhempy (15min)...', 1);
+    $hash->{installing} = 0;
+    InternalTimer(gettimeofday()+2, 'BindingsIo_installing', $hash, 0);
+  }
 
-  return undef;
+  if ($init_done)
+  {
+    # put in fhempy room
+    if (AttrVal($name, q[room], q[]) eq q[]) {
+      CommandAttr(undef, qq[$name room fhempy]);
+    }
+    # set icon
+    if (AttrVal($name, q[icon], q[]) eq q[]) {
+      CommandAttr(undef, qq[$name icon file_json-ld2]);
+    }
+    # set group
+    if (AttrVal($name, q[group], q[]) eq q[]) {
+      CommandAttr(undef, qq[$name group fhempy]);
+    }
+    # set devStateIcon
+    my $devstateicon_val = AttrVal($name, q[devStateIcon], q[]);
+    if ($devstateicon_val eq q[] or index($devstateicon_val, "ver_available") == -1) {
+      my $devstate_cmd = '{
+        my $status_img = "10px-kreis-gruen";;
+        my $status_txt = "connected";;
+        my $ver = ReadingsVal($name, "version", "-");;
+        my $ver_available = ReadingsVal($name, "version_available", $ver);;
+        my $update_icon = "";;
+        if ($ver_available ne $ver) {
+          $status_img = "10px-kreis-gelb";;
+          $status_txt = "Version ".$ver_available." available for update";;
+        }
+        if (ReadingsVal($name, "state", "disconnected") eq "disconnected") {
+          $status_img = "10px-kreis-rot";;
+          $status_txt = "disconnected";;
+        }
+        $update_icon = "<a  href=\"/fhem?cmd.dummy=set $name update&XHR=1\" title=\"Start ".$ver_available." update\">".FW_makeImage("refresh")."</a>";;
+        "<div><a>".FW_makeImage($status_img, $status_txt)."</a><a> ".$ver." </a>".$update_icon."</div>"
+      }';
+      $devstate_cmd =~ tr/\n//d;
+      CommandAttr(undef, "$name devStateIcon $devstate_cmd");
+    }
+  }
+
+  return ;
+}
+
+sub
+BindingsIo_initFrame($) {
+  my ($hash) = @_;
+  $hash->{frame} = Protocol::WebSocket::Frame->new;
+  #$hash->{frame}->{max_fragments_amount} = 1000;
+  $hash->{frame}->{max_payload_size} = 0;
+}
+
+sub
+BindingsIo_installing($) {
+  my ($hash) = @_;
+  my $state_reading = ReadingsVal($hash->{NAME}, "version", "");
+  if ($state_reading ne "") {
+    return undef;
+  }
+  if ($hash->{installing} == 0) {
+    readingsSingleUpdate($hash, "state", "Installing fhempy (15min).", 1);
+  } elsif ($hash->{installing} == 1) {
+    readingsSingleUpdate($hash, "state", "Installing fhempy (15min)..", 1);
+  } elsif ($hash->{installing} == 2) {
+    readingsSingleUpdate($hash, "state", "Installing fhempy (15min)...", 1);
+  }
+  $hash->{installing} = ($hash->{installing} + 1) % 3;
+  InternalTimer(gettimeofday()+2, "BindingsIo_installing", $hash, 0);
 }
 
 sub
 BindingsIo_connectDev($) {
   my ($hash) = @_;
+  BindingsIo_setIODevAttr($hash);
   DevIo_CloseDev($hash) if(DevIo_IsOpen($hash));
   DevIo_OpenDev($hash, 0, "BindingsIo_doInit", "BindingsIo_Callback");
 }
 
+sub
+BindingsIo_setIODevAttr($) {
+  my ($hash) = @_;
+
+  my $fhempydev_str = BindingsIo_getIODevList($hash);
+
+  foreach my $fhem_dev (sort keys %main::defs) {
+    my $devhash = $main::defs{$fhem_dev};
+    if(defined($devhash->{"FHEMPYTYPE"})) {
+      my $attr_list = $devhash->{".AttrList"};
+      $attr_list =~ s/IODev/IODev:$fhempydev_str/;
+      setDevAttrList($devhash->{NAME}, $attr_list);
+    }
+  }
+}
+
+sub
+BindingsIo_getIODevList($) {
+  my @fhempydev_list = ();
+  foreach my $fhem_dev (sort keys %main::defs) {
+    push(@fhempydev_list, $fhem_dev) if ($main::defs{$fhem_dev}->{TYPE} eq "BindingsIo");
+  }
+  my $fhempydev_str = join(",", @fhempydev_list);
+  return $fhempydev_str;
+}
+
 sub
 BindingsIo_doInit($) {
   my ($hash) = @_;
 
+  $hash->{connecttime} = time;
+
+  BindingsIo_initFrame($hash);
+
   # initialize all devices (send Define)
   my $bindingType = uc($hash->{BindingType})."TYPE";
   foreach my $fhem_dev (sort keys %main::defs) {
@@ -129,7 +249,6 @@ sub
 BindingsIo_Notify($)
 {
   my ($hash, $dev) = @_;
-  return if($dev->{NAME} ne "global");
 
   return "" if(IsDisabled($hash->{NAME})); # Return without any further action if the module is disabled
 
@@ -141,10 +260,13 @@ BindingsIo_Notify($)
   foreach my $event (@{$events}) {
     $event = "" if(!defined($event));
 
-    if ($event eq "INITIALIZED") {
+    if ($dev->{NAME} eq "global" && $event eq "INITIALIZED") {
       InternalTimer(gettimeofday()+5, "BindingsIo_connectDev", $hash, 0);
-    } elsif ($event eq "UPDATE") {
+    } elsif ($dev->{NAME} eq "global" && $event eq "UPDATE") {
       BindingsIo_Write($hash, $hash, "update", [], {});
+      Log3 $hash, 1, "BindingsIo ($hash->{NAME}): ==> FHEMPY UPDATE STARTED...CHECK FHEMPY STATE FOR STATUS <==";
+    } else {
+      #BindingsIo_Write($hash, $dev, "event", [$event], {});
     }
   }
 
@@ -155,8 +277,13 @@ sub
 BindingsIo_Callback($$) {
   my ($hash, $error) = @_;
   my $name = $hash->{NAME};
-  if (defined($error)) {
-    Log3 $name, 1, "BindingsIo: ERROR $name - error while connecting: $error"; 
+  if (!defined($hash->{prev_error})) {
+    $hash->{prev_error} = "";
+  }
+
+  if (defined($error) and $hash->{prev_error} ne $error) {
+    Log3 $name, 1, "BindingsIo ($hash->{NAME}): ERROR during connection setup: $error";
+    $hash->{prev_error} = $error;
   }
 }
 
@@ -189,15 +316,22 @@ BindingsIo_Write($$$$$) {
     $function = "Define";
   }
 
-  if($hash->{STATE} eq "disconnected" || !DevIo_IsOpen($hash)) {
-    if ($init_done == 1) {
-      readingsSingleUpdate($devhash, "state", $hash->{BindingType}."Binding offline", 1);
+  if($function ne "event" && ($hash->{STATE} eq "disconnected" || !DevIo_IsOpen($hash))) {
+    if ($init_done == 1 && $hash->{NAME} ne $devhash->{NAME}) {
+      readingsSingleUpdate($devhash, "state", $hash->{BindingType}." server offline", 1);
+
+      my $attr_list = $devhash->{".AttrList"};
+      if ($attr_list eq ".*") {
+        my $fhempydev_str = BindingsIo_getIODevList($hash);
+        $attr_list = "IODev:$fhempydev_str";
+        setDevAttrList($devhash->{NAME}, $attr_list);
+      }
     }
     return undef;
   }
 
   my $waitingForId = int(rand()*100000000);
-  Log3 $hash, 4, "BindingsIo: start ".$hash->{BindingType}."Function: ".$devhash->{NAME}." => $function ($waitingForId)";
+  Log3 $hash, 4, "BindingsIo ($hash->{NAME}): start ".$hash->{BindingType}."Function: ".$devhash->{NAME}." => $function ($waitingForId)";
 
   my $bindingType = uc($hash->{BindingType})."TYPE";
 
@@ -212,14 +346,24 @@ BindingsIo_Write($$$$$) {
     "defargsh" => $devhash->{argsh}
   );
   $msg{$bindingType} =  $devhash->{$bindingType};
+  # keep this for one year (written on 11.10.2021)
+  if ($bindingType eq "FHEMPYTYPE") {
+    $msg{PYTHONTYPE} = $devhash->{$bindingType};
+  }
 
   if ($function eq "update") {
     $msg{"msgtype"} = "update";
     $waitforresponse = 0;
+  } elsif ($function eq "restart") {
+    $msg{"msgtype"} = "restart";
+    $waitforresponse = 0;
+  } elsif ($function eq "event") {
+    $msg{"msgtype"} = "event";
+    $waitforresponse = 0;
   }
 
-  my $utf8msg = Encode::encode("utf-8", Encode::decode("utf-8", to_json(\%msg)));
-  Log3 $hash, 4, "BindingsIo: <<< WS: ".$utf8msg;
+  my $utf8msg = to_json(\%msg);
+  Log3 $hash, 4, "BindingsIo ($hash->{NAME}): <<< WS: ".$utf8msg;
   if (length $utf8msg > 0) {
     DevIo_SimpleWrite($hash, $utf8msg, 0);
     if ($waitforresponse == 0) {
@@ -227,43 +371,60 @@ BindingsIo_Write($$$$$) {
     }
   }
 
-  my $py_timeout = 2500;
+  my $py_timeout = 3000;
+  my $cur_time = time;
+  if (($cur_time - $hash->{connecttime}) < 120) {
+    $py_timeout = 60000;
+  }
   if ($function eq "Define" or $init_done == 0 or $initrun == 1) {
     # wait 10s on Define, this might happen on startup
-    $py_timeout = 10000;
+    $py_timeout = 30000;
   }
   my $returnval = "";
   my $t1 = time * 1000;
   while (1) {
-    if (!DevIo_IsOpen($hash)) {
-      Log3 $hash, 1, "BindingsIo: ERROR: Connection closed while waiting for function to finish (id: $waitingForId)";
+    if (not defined(DevIo_IsOpen($hash))) {
+      Log3 $hash, 1, "BindingsIo ($hash->{NAME}): WARNING: Connection closed while waiting for function to finish (id: $waitingForId)";
       last;
     }
     my $t2 = time * 1000;
     if (($t2 - $t1) > $py_timeout) {
       $timeouts = $timeouts + 1;
-      Log3 $hash, 1, "BindingsIo: ERROR: Timeout while waiting for function to finish (id: $waitingForId)";
-      readingsSingleUpdate($devhash, "state", $hash->{BindingType}."Binding timeout", 1);
-      $returnval = ""; # was before "Timeout while waiting for reply from $function"
-      if ($timeouts > 1) {
-        # SimpleRead will close the connection and DevIo reconnect starts
-        Log3 $hash, 1, "BindingsIo: ERROR: Too many timeouts, disconnect now and try to reconnect";
-        DevIo_Disconnected($hash);
+      Log3 $hash, 1, "BindingsIo ($hash->{NAME}): ERROR: Timeout while waiting for function to finish (id: $waitingForId)";
+      while (my ($key, $value) = each (%msg))
+      {
+        Log3 $hash, 1, "  $key =>  $msg{$key}";
       }
       last;
+      readingsSingleUpdate($devhash, "state", $hash->{BindingType}." timeout", 1);
+      $returnval = ""; # was before "Timeout while waiting for reply from $function"
+      #if ($timeouts > 1) {
+        # SimpleRead will close the connection and DevIo reconnect starts
+      #  Log3 $hash, 1, "BindingsIo ($hash->{NAME}): ERROR: Too many timeouts, disconnect now and try to reconnect";
+      #  DevIo_Disconnected($hash);
+      #}
+      last;
     }
     
     $returnval = BindingsIo_readWebsocketMessage($hash, $devhash, $waitingForId, 0);
     if ($returnval ne "empty" && $returnval ne "continue") {
       $timeouts = 0;
       last;
+    } else {
+      $returnval = "";
     }
   }
-  Log3 $hash, 4, "BindingsIo: end ".$hash->{BindingType}."Function: ".$devhash->{NAME}." => $function ($waitingForId) - result: ".$returnval;
+  Log3 $hash, 4, "BindingsIo ($hash->{NAME}): end ".$hash->{BindingType}."Function: ".$devhash->{NAME}." => $function ($waitingForId) - result: ".$returnval;
 
-  if ($returnval eq "") {
+  if ($returnval eq "" || $returnval eq "continue") {
     $returnval = undef;
   }
+
+  my $cnt = 0;
+  while ($hash->{ReceiverQueue}->pending() > 0 && $cnt < 50) {
+    BindingsIo_readWebsocketMessage($hash, undef, 0, 1);
+    $cnt = $cnt + 1
+  }
   
   return $returnval;
 }
@@ -292,10 +453,12 @@ BindingsIo_Set($$$)
 {
   my ($hash, $a, $h) = @_;
   my $cmd = @$a[1];
-  my $list = "update:noArg";
+  my $list = "update:noArg restart:noArg";
 
   if ($cmd eq 'update') {
     return BindingsIo_Write($hash, $hash, "update", [], {});
+  } elsif ($cmd eq 'restart') {
+    return BindingsIo_Write($hash, $hash, "restart", [], {});
   }
 
   return "Unknown argument $cmd, choose one of $list";
@@ -331,12 +494,14 @@ BindingsIo_Shutdown($)
 
 sub BindingsIo_processMessage($$$$) {
   my ($hash, $devhash, $waitingForId, $response) = @_;
-  $response = Encode::encode("UTF-8", $response);
+  $response = Encode::encode("utf8", $response);
   Log3 $hash, 5, "processMessage: ".$response;
   my $json = eval {from_json($response)};
   if ($@) {
-    Log3 $hash, 1, "BindingsIo: ERROR JSON: ".$@;
-    Log3 $hash, 1, "BindingsIo: received JSON was: ".$response;
+    Log3 $hash, 1, "BindingsIo ($hash->{NAME}): ERROR JSON: ".$@;
+    Log3 $hash, 1, "BindingsIo ($hash->{NAME}): received JSON was: ".$response;
+    # reset frames
+    BindingsIo_initFrame($hash);
     return "error";
   }
 
@@ -355,7 +520,7 @@ sub BindingsIo_processMessage($$$$) {
         return $json->{error};
       }
       if ($devhash->{NAME} ne $json->{NAME}) {
-        Log3 $hash, 1, "BindingsIo: ERROR: Received wrong WS message, waiting for ".$devhash->{NAME}.", but received ".$json->{NAME};
+        Log3 $hash, 1, "BindingsIo ($hash->{NAME}): ERROR: Received wrong WS message, waiting for ".$devhash->{NAME}.", but received ".$json->{NAME};
         return "nothandled";
       } else {
         foreach my $key (keys %$json) {
@@ -366,8 +531,8 @@ sub BindingsIo_processMessage($$$$) {
         $returnval = $json->{returnval};
       }
     } else {
-      Log3 $hash, 4, "BindingsIo: Received message doesn't match, continue waiting...";
-      Log3 $hash, 4, "BindingsIo:   received id (".$json->{id}.") = waiting for id (".$waitingForId.")";
+      Log3 $hash, 4, "BindingsIo ($hash->{NAME}): Received message doesn't match, continue waiting...";
+      Log3 $hash, 4, "BindingsIo ($hash->{NAME}):   received id (".$json->{id}.") = waiting for id (".$waitingForId.")";
       return "nothandled";
     }
   } elsif ($json->{msgtype} eq "update_hash") {
@@ -379,13 +544,23 @@ sub BindingsIo_processMessage($$$$) {
       $devhash->{$key} = $json->{$key};
     }
   } elsif ($json->{msgtype} eq "version") {
-    readingsSingleUpdate($hash, "version", $json->{version}, 1);
+    foreach my $key (keys %$json) {
+      if ($key eq "msgtype") {
+        next;
+      }
+      readingsSingleUpdate($hash, $key, $json->{$key}, 1);
+    }
   } elsif ($json->{msgtype} eq "command") {
     my $ret = 0;
     my %res;
+    # set proper IODev list for easier handling
+    if ($json->{command} =~ /^setDevAttrList/) {
+        my $liststr = BindingsIo_getIODevList($hash);
+        $json->{command} =~ s/IODev/IODev:$liststr/;
+    }
     $ret = eval $json->{command};
     if ($@) {
-      Log3 $hash, 1, "BindingsIo: ERROR failed (".$json->{command}."): ".$@;
+      Log3 $hash, 1, "BindingsIo ($hash->{NAME}): ERROR failed (".$json->{command}."): ".$@;
       %res = (
         awaitId => $json->{awaitId},
         error => 1,
@@ -400,7 +575,7 @@ sub BindingsIo_processMessage($$$$) {
       );
     }
     my $utf8msg = Encode::encode("utf-8", Encode::decode("utf-8", to_json(\%res)));
-    Log3 $hash, 4, "BindingsIo: <<< WS: ".$utf8msg;
+    Log3 $hash, 4, "BindingsIo ($hash->{NAME}): <<< WS: ".$utf8msg;
     if (length $utf8msg > 0) {
       DevIo_SimpleWrite($hash, $utf8msg, 0);
     }
@@ -430,6 +605,10 @@ sub BindingsIo_SimpleReadWithTimeout($$) {
         my $bufws = DevIo_DecodeWS($hash, $buf) if($hash->{WEBSOCKET});
         return $bufws;
       } else {
+        my $op = (ord(substr($buf,0,1)) & 0x0F);
+        if ($op == 8) {
+          return undef;
+        }
         return $buf;
       }
     }
@@ -444,21 +623,21 @@ sub BindingsIo_readWebsocketMessage($$$$) {
   my $returnval = "continue";
   my $response = "";
   if (defined($socketready) && $socketready == 1) {
-    Log3 $hash, 5, "BindingsIo: DevIo_SimpleRead";
+    Log3 $hash, 5, "BindingsIo ($hash->{NAME}): DevIo_SimpleRead";
     if ($USE_DEVIO_DECODEWS == 0) {
       delete $hash->{WEBSOCKET};
     }
     $response = BindingsIo_SimpleReadWithTimeout($hash, 0.00001);
     #$response = DevIo_SimpleRead($hash);
     $hash->{WEBSOCKET} = 1;
-    Log3 $hash, 5, "BindingsIo: DevIo_SimpleRead NoTimeout";
+    Log3 $hash, 5, "BindingsIo ($hash->{NAME}): DevIo_SimpleRead NoTimeout";
   } else {
-    Log3 $hash, 5, "BindingsIo: DevIo_SimpleRead";
+    Log3 $hash, 5, "BindingsIo ($hash->{NAME}): DevIo_SimpleRead";
     $response = BindingsIo_SimpleReadWithTimeout($hash, 0.01);
-    Log3 $hash, 5, "BindingsIo: DevIo_SimpleRead WithTimeout";
+    Log3 $hash, 5, "BindingsIo ($hash->{NAME}): DevIo_SimpleRead WithTimeout";
   }
   if (defined($response) && $response eq "connectionclosed") {
-    Log3 $hash, 5, "BindingsIo: DevIo_SimpleRead WithTimeout - connection seems to be closed";
+    Log3 $hash, 5, "BindingsIo ($hash->{NAME}): DevIo_SimpleRead WithTimeout - connection seems to be closed";
     # connection seems to be closed, call simpleread to disconnect
     # connection will be reopened by ReadyFn
     DevIo_SimpleRead($hash);
@@ -468,7 +647,7 @@ sub BindingsIo_readWebsocketMessage($$$$) {
   if ($USE_DEVIO_DECODEWS == 0) {
     $hash->{frame}->append($response);
     while (my $r = $hash->{frame}->next) {
-      Log3 $hash, 4, "BindingsIo: >>> WS: ".$r;
+      Log3 $hash, 4, "BindingsIo ($hash->{NAME}): >>> WS: ".$r;
       my $resTemp = {
         "response" => $r,
         "time" => time
@@ -477,7 +656,7 @@ sub BindingsIo_readWebsocketMessage($$$$) {
     }
   } else {
     if (defined($response) && $response ne "") {
-      Log3 $hash, 4, "BindingsIo: >>> WS: ".$response;
+      Log3 $hash, 4, "BindingsIo ($hash->{NAME}): >>> WS: ".$response;
       my $resTemp = {
         "response" => $response,
         "time" => time
@@ -488,7 +667,7 @@ sub BindingsIo_readWebsocketMessage($$$$) {
 
   # handle messages on the queue
   $hash->{TempReceiverQueue} = Thread::Queue->new();
-  Log3 $hash, 5, "BindingsIo: QUEUE: start handling - ".$hash->{ReceiverQueue}->pending();
+  Log3 $hash, 5, "BindingsIo ($hash->{NAME}): QUEUE: start handling - ".$hash->{ReceiverQueue}->pending();
   while (my $msg = $hash->{ReceiverQueue}->dequeue_nb()) {
     if ((time - $msg->{'time'}) > 10) {
       next;
@@ -502,12 +681,14 @@ sub BindingsIo_readWebsocketMessage($$$$) {
       $hash->{TempReceiverQueue}->enqueue($msg);
     }
   }
-  Log3 $hash, 5, "BindingsIo: QUEUE: finished handling - ".$hash->{ReceiverQueue}->pending();
-
+  
   # add not matching messages to the queue
   while (my $msg = $hash->{TempReceiverQueue}->dequeue_nb()) {
     $hash->{ReceiverQueue}->enqueue($msg);
   }
+
+  Log3 $hash, 5, "BindingsIo ($hash->{NAME}): QUEUE: finished handling - ".$hash->{ReceiverQueue}->pending();
+
   return $returnval;
 }
 
@@ -526,7 +707,7 @@ sub BindingsIo_readWebsocketMessage($$$$) {
   <a name="BindingsIo_Define"></a>
   <b>Define</b>
   <ul>
-  define pybinding BindingsIo Python
+  define pybinding BindingsIo fhempy
   </ul>
 
 </ul><br>
