@@ -1,11 +1,13 @@
 
-# $Id: 10_PythonBinding.pm 18283 2019-01-16 16:58:23Z dominikkarall $
+# $Id: 10_PythonBinding.pm 18283 2019-01-16 16:58:23Z fhempy $
 
 package main;
 
 use strict;
 use warnings;
 
+use CoProcess;
+
 sub Log($$);
 sub Log3($$$);
 
@@ -39,7 +41,7 @@ sub PythonBinding_detailFn($$$$)
 
   my $logfile = AttrVal($name, 'logfile', 'FHEM' );
   if( $logfile && $logfile ne "FHEM" ) {
-    my $name = 'PythonBindingLog';
+    my $name = 'fhempy_log';
     $ret .= "<a href=\"$FW_ME?detail=$name\">fhempy Logfile Viewer</a><br>";
   }
 
@@ -59,6 +61,9 @@ sub PythonBinding_Define($$$)
 
   Log3 $hash, 3, "PythonBinding v1.0.0";
 
+  # automatically migrate to fhempyServer
+  $hash->{TYPE} = "fhempyServer";
+
   $hash->{logfile} = "./log/fhempy-%Y-%m-%d.log";
   $hash->{CoProcess} = {  name => 'fhempy',
                           cmdFn => 'PythonBinding_getCmd',
@@ -72,7 +77,7 @@ sub PythonBinding_Define($$$)
   }
 
   # put in fhempy room
-  CommandAttr(undef, "$name room fhempy");
+  CommandAttr(undef, "$name room fhempy") if( !AttrVal($name, 'room', undef ) );;
   CommandAttr(undef, "$name nrarchive 10") if( !AttrVal($name, 'nrarchive', undef ) );
 
   if( $attr{global}{logdir} ) {
@@ -118,7 +123,7 @@ sub PythonBinding_Undefine($$)
     $hash->{reason} = 'delete';
     CoProcess::stop($hash);
       
-    return "$name will be deleted after pythonbinding.py has stopped or after 5 seconds. whatever comes first.";
+    return "$name will be deleted after fhempy has stopped or after 5 seconds. whatever comes first.";
   }   
       
   delete $modules{$hash->{TYPE}}{defptr};
@@ -147,11 +152,11 @@ sub PythonBinding_Attr($$$)
 
   if( $attrName eq 'logfile' ) {
     if( $cmd eq "set" && $attrVal && $attrVal ne 'FHEM' ) {
-      fhem( "defmod -temporary PythonBindingLog FileLog $attrVal fakelog" );
-      CommandAttr( undef, 'PythonBindingLog room fhempy' );
+      fhem( "defmod fhempy_log FileLog $attrVal Logfile" );
+      CommandAttr( undef, 'fhempy_log room fhempy' ) if( !AttrVal($name, 'room', undef ) );
       $hash->{logfile} = $attrVal;
     } else {
-      fhem( "delete PythonBindingLog" );
+      fhem( "delete fhempy_log" );
     }
 
     $attr{$name}{$attrName} = $attrVal;
