@@ -81,14 +81,15 @@ class BTLEConnection(btle.DefaultDelegate):
         self._mac = mac
         self._callbacks = {}
         self._keep_connected = keep_connected
-        self._connection_etablished_callback = connection_established_callback
+        self._connection_established_callback = connection_established_callback
 
     def set_keep_connected(self, new_state):
         self._keep_connected = new_state
-        if new_state == False:
+        if new_state is False:
             if self._conn:
                 try:
                     self._conn.disconnect()
+                    self._conn = None
                 except Exception:
                     pass
 
@@ -137,8 +138,8 @@ class BTLEConnection(btle.DefaultDelegate):
                     self._conn.connect(
                         self._mac, iface=self._ifaces[self._iface_idx], timeout=10
                     )
-                    if self._connection_etablished_callback is not None:
-                        self._connection_etablished_callback(self._mac)
+                    if self._connection_established_callback is not None:
+                        self._connection_established_callback(self._mac)
                     break
                 except btle.BTLEException as ex:
                     _LOGGER.debug(
@@ -151,8 +152,8 @@ class BTLEConnection(btle.DefaultDelegate):
                         self._conn.connect(
                             self._mac, iface=self._ifaces[self._iface_idx], timeout=10
                         )
-                        if self._connection_etablished_callback is not None:
-                            self._connection_etablished_callback(self._mac)
+                        if self._connection_established_callback is not None:
+                            self._connection_established_callback(self._mac)
                         break
                     except Exception as ex2:
                         _LOGGER.debug(
@@ -182,6 +183,10 @@ class BTLEConnection(btle.DefaultDelegate):
             for callback in self._callbacks[handle]:
                 callback(data)
 
+        if "all" in self._callbacks:
+            for callback in self._callbacks["all"]:
+                callback(data)
+
     @property
     def mac(self):
         """Return the MAC address of the connected device."""
@@ -197,6 +202,16 @@ class BTLEConnection(btle.DefaultDelegate):
         with self:
             return self._conn.readCharacteristic(handle)
 
+    def disconnect(self):
+        if self._conn:
+            self._conn.disconnect()
+            self._conn = None
+
+    def stop_helper(self):
+        self._conn._helper.kill()
+        self._conn._helper = None
+        self._conn = None
+
     def write_characteristic(
         self, handle, value, timeout=DEFAULT_TIMEOUT, with_response=True
     ):
