@@ -1,7 +1,8 @@
 import asyncio
 import socket
 
-from zeroconf import ServiceBrowser, ServiceInfo, Zeroconf
+from zeroconf import Zeroconf
+from zeroconf.asyncio import AsyncServiceInfo, AsyncZeroconf
 
 
 class zeroconf:
@@ -17,12 +18,13 @@ class zeroconf:
     def __init__(self, logger):
         self.logger = logger
         self.loop = asyncio.get_event_loop()
+        self.async_zeroconf = AsyncZeroconf()
         self.zeroconf = Zeroconf()
 
     async def register_service(self, type, name, port, properties):
         hostname = socket.gethostname()
         local_ip = socket.gethostbyname(hostname)
-        info = ServiceInfo(
+        info = AsyncServiceInfo(
             type + "._tcp.local.",
             name + "." + type + "._tcp.local.",
             addresses=[socket.inet_aton(local_ip)],
@@ -30,14 +32,17 @@ class zeroconf:
             properties=properties,
             server=hostname + ".local.",
         )
-        self.zeroconf.register_service(info)
+        await self.async_zeroconf.async_register_service(info)
         return info
 
     async def unregister_service(self, info):
-        self.zeroconf.unregister_service(info)
+        await self.async_zeroconf.async_unregister_service(info)
 
     def get_zeroconf(self):
         return self.zeroconf
 
-    def stop(self):
-        self.zeroconf.close()
\ No newline at end of file
+    def get_async_zeroconf(self):
+        return self.async_zeroconf
+
+    async def stop(self):
+        await self.async_zeroconf.async_close()
