@@ -1,11 +1,8 @@
 import asyncio
-import concurrent.futures
-import logging
-import threading
 import traceback
 
 from fhempy.lib.generic import FhemModule
-from zeroconf import ServiceBrowser, Zeroconf
+from zeroconf.asyncio import AsyncServiceBrowser
 
 from .. import fhem
 from ..core.zeroconf import zeroconf
@@ -21,9 +18,8 @@ class discover_mdns(FhemModule):
 
     # zeroconf callback
     def update_service(self, zeroconf, type, name):
-        info = zeroconf.get_service_info(type, name)
-        self.logger.debug("Service %s updated, service info: %s" % (name, info))
-        res = asyncio.run_coroutine_threadsafe(self.foundDevice(name, info), self.loop)
+        self.logger.debug("Service %s updated" % (name))
+        res = asyncio.run_coroutine_threadsafe(self.foundDevice(type, name), self.loop)
         res.result()
 
     # zeroconf callback
@@ -32,15 +28,13 @@ class discover_mdns(FhemModule):
 
     # zeroconf callback
     def add_service(self, zeroconf, type, name):
-        info = zeroconf.get_service_info(type, name)
-        self.logger.debug("Service %s added, service info: %s" % (name, info))
-        res = asyncio.run_coroutine_threadsafe(self.foundDevice(name, info), self.loop)
+        self.logger.debug("Service %s added" % (name))
+        res = asyncio.run_coroutine_threadsafe(self.foundDevice(type, name), self.loop)
         res.result()
 
-    async def foundDevice(self, name, info):
-        if info is None:
-            return
+    async def foundDevice(self, type, name):
         try:
+            info = await self.zeroconf.async_get_service_info(type, name)
 
             def get_value(key):
                 """Retrieve value and decode to UTF-8."""
@@ -67,7 +61,7 @@ class discover_mdns(FhemModule):
                         get_value("md").replace(" ", "_")
                         + "_"
                         + get_value("fn").replace(" ", "_")
-                        + " PythonModule googlecast '"
+                        + " fhempy googlecast '"
                         + get_value("fn")
                         + "'",
                     )
@@ -108,7 +102,7 @@ class discover_mdns(FhemModule):
                 ):
                     self.logger.debug("create spotify")
                     await fhem.CommandDefine(
-                        self.hash, "spotify_connect PythonModule spotify"
+                        self.hash, "spotify_connect fhempy spotify"
                     )
                 else:
                     self.logger.debug("device spotify exists already, do not create")
@@ -117,13 +111,13 @@ class discover_mdns(FhemModule):
 
             # wait for the devices to initialize
             await asyncio.sleep(10)
-        except Exception as err:
-            self.logger.error(traceback.print_exc())
+        except Exception:
+            self.logger.exception("Failed to handle foundDevice")
 
     async def runZeroconfScan(self):
         # await here to finish define before zeroconf object is created
         await asyncio.sleep(1)
-        self.zeroconf = zeroconf.get_instance(self.logger).get_zeroconf()
+        self.zeroconf = zeroconf.get_instance(self.logger).get_async_zeroconf()
         listener = self
         services = [
             "_googlecast._tcp.local.",
@@ -131,7 +125,7 @@ class discover_mdns(FhemModule):
             "_http._tcp.local.",
             "_spotify-connect",
         ]
-        self.browser = ServiceBrowser(self.zeroconf, services, listener)
+        self.browser = AsyncServiceBrowser(self.zeroconf.zeroconf, services, listener)
 
     # FHEM
     async def Define(self, hash, args, argsh):
