@@ -6,10 +6,10 @@ import logging
 import time
 from datetime import timedelta
 
-import aiohttp
-from async_upnp_client.aiohttp import (AiohttpNotifyServer,
-                                       AiohttpSessionRequester, get_local_ip)
+from aiohttp import ClientError, ClientSession
+from async_upnp_client.aiohttp import AiohttpNotifyServer, AiohttpSessionRequester
 from async_upnp_client.profiles.dlna import DeviceState, DmrDevice
+from async_upnp_client.utils import async_get_local_ip
 from fhempy.lib.generic import FhemModule
 
 from .. import fhem
@@ -29,7 +29,7 @@ def catch_request_errors():
             """Catch asyncio.TimeoutError, aiohttp.ClientError errors."""
             try:
                 return await func(self, *args, **kwargs)
-            except (asyncio.TimeoutError, aiohttp.ClientError):
+            except (asyncio.TimeoutError, ClientError):
                 self.logger.error("Error during call %s", func.__name__)
 
         return wrapper
@@ -79,10 +79,14 @@ class dlna_dmr(FhemModule):
 
         self.upnp_device = upnp_device
         # build upnp/aiohttp requester
-        session = aiohttp.ClientSession()
+        session = ClientSession()
         requester = AiohttpSessionRequester(session, True)
         # ensure event handler has been started
-        server_host = get_local_ip()
+        server_host = await async_get_local_ip()
+        if server_host:
+            server_host = server_host[1]
+        else:
+            self.logger.error(f"Failed to add device {self.device}")
         server_port = DEFAULT_LISTEN_PORT
         event_handler = await self.async_start_event_handler(
             server_host, server_port, requester
@@ -126,7 +130,6 @@ class dlna_dmr(FhemModule):
 
     # FHEM Function
     async def Undefine(self, hash):
-        await super().Undefine(hash)
         await ssdp.getInstance(self.logger).stop_search()
         if self.server:
             await self.server.stop_server()
@@ -134,13 +137,14 @@ class dlna_dmr(FhemModule):
             await self.device.cleanup()
             del self.device
             self.device = None
+        await super().Undefine(hash)
 
     # FHEM Function
     async def Define(self, hash, args, argsh):
         """Set up DLNA DMR platform."""
         await super().Define(hash, args, argsh)
         if len(args) < 4:
-            return "Usage: define device PythonModule dlna_dmr <UUID>"
+            return "Usage: define device fhempy dlna_dmr <UUID>"
 
         if await fhem.AttrVal(self.hash["NAME"], "icon", "") == "":
             await fhem.CommandAttr(self.hash, self.hash["NAME"] + " icon scene_scene")
