@@ -8,14 +8,13 @@ from enum import IntEnum
 
 from dbus import DBusException
 
-from .. import fhem, utils
-from ..generic import FhemModule
+from .. import fhem, generic, utils
 from . import eq3btsmart as eq3
 from .connection import BTLEConnection
 
 
 class Mode(IntEnum):
-    """ Thermostat modes. """
+    """Thermostat modes."""
 
     Unknown = -1
     Closed = 0
@@ -30,7 +29,7 @@ class Mode(IntEnum):
 # TODO set windowOpen, windowOpenTime, eco/comfortTemperature
 
 
-class eq3bt(FhemModule):
+class eq3bt(generic.FhemModule):
     def __init__(self, logger):
         super().__init__(logger)
 
@@ -40,7 +39,12 @@ class eq3bt(FhemModule):
                 "format": "str",
                 "options": "on,off",
                 "help": "On...keeps bluetooth low energy connection active which makes commands to be executed immediately",
-            }
+            },
+            "max_retries": {
+                "default": 5,
+                "format": "int",
+                "help": "Maximum retries for connection setup, default=5.",
+            },
         }
         self.set_attr_config(attr_list)
 
@@ -73,12 +77,12 @@ class eq3bt(FhemModule):
             "windowOpenTemperature": {
                 "args": ["temp"],
                 "params": {"temp": {"format": "float"}},
-                "options": "slider,4.5,0.5,30,1",
+                "options": "slider,5,0.5,30,1",
             },
             "windowOpenTime": {
-                "args": ["seconds"],
-                "params": {"seconds": {"format": "int"}},
-                "options": "slider,0,1,3600,1",
+                "args": ["minutes"],
+                "params": {"minutes": {"format": "int"}},
+                "options": "slider,0,5,60",
             },
             "ecoTemperature": {
                 "args": ["temp"],
@@ -95,13 +99,14 @@ class eq3bt(FhemModule):
 
         self._last_update = 0
         self._mac = None
+        self.thermostat = None
 
     # FHEM FUNCTION
     async def Define(self, hash, args, argsh):
         await super().Define(hash, args, argsh)
         self.hash = hash
         if len(args) < 4:
-            return "Usage: define eq3_livingroom PythonModule eq3bt <MAC>"
+            return "Usage: define eq3_livingroom fhempy eq3bt <MAC>"
         self._mac = args[3]
         self.hash["MAC"] = self._mac
         self.logger.info(f"Define: eq3bt {self._mac}")
@@ -120,6 +125,7 @@ class eq3bt(FhemModule):
                 self.logger,
                 self._mac,
                 keep_connection=self._attr_keep_connected == "on",
+                max_retries=self._attr_max_retries,
             )
         except DBusException:
             dbus_conf_err = (
@@ -145,6 +151,11 @@ class eq3bt(FhemModule):
         self.create_async_task(self.consumption_rotate())
         return ""
 
+    async def Undefine(self, hash):
+        if self.thermostat:
+            self.thermostat.disconnect()
+        return await super().Undefine(hash)
+
     def seconds_till_midnight(self):
         """Get the number of seconds until midnight."""
         n = datetime.now()
@@ -423,7 +434,7 @@ class eq3bt(FhemModule):
         self.create_async_task(
             self.set_and_update(
                 functools.partial(
-                    self.thermostat.set_windows_open_config,
+                    self.thermostat.set_window_open_config,
                     params["temp"],
                     duration_sec,
                 )
@@ -436,7 +447,7 @@ class eq3bt(FhemModule):
         self.create_async_task(
             self.set_and_update(
                 functools.partial(
-                    self.thermostat.set_windows_open_config, temp, params["seconds"]
+                    self.thermostat.set_window_open_config, temp, params["minutes"] * 60
                 )
             )
         )
@@ -488,10 +499,22 @@ class eq3bt(FhemModule):
 
 
 class FhemThermostat(eq3.Thermostat):
-    def __init__(self, logger, mac, keep_connection):
+    def __init__(self, logger, mac, keep_connection, max_retries):
         self.logger = logger
         self._keep_conection = keep_connection
-        super(FhemThermostat, self).__init__(mac, BTLEConnection, keep_connection=True)
+        self._max_retries = max_retries
+        super(FhemThermostat, self).__init__(
+            mac,
+            BTLEConnection,
+            keep_connection=self._keep_conection,
+            max_retries=self._max_retries,
+        )
+
+    def disconnect(self):
+        super().set_keep_connected(False)
+
+    def set_max_retries(self, max_retries):
+        super().set_max_retries(max_retries)
 
     def set_keep_connection(self, new_state):
         self.set_keep_connected(new_state)
@@ -508,7 +531,7 @@ class FhemThermostat(eq3.Thermostat):
     def set_temperature_offset(self, temp):
         self.temperature_offset = temp
 
-    def set_windows_open_config(self, temperature, duration):
+    def set_window_open_config(self, temperature, duration):
         self.window_open_config(temperature, duration)
 
     def set_target_temperature(self, temp):
