@@ -48,7 +48,7 @@ EQ3BT_ON_TEMP = 30.0
 
 
 class Mode(IntEnum):
-    """ Thermostat modes. """
+    """Thermostat modes."""
 
     Unknown = -1
     Closed = 0
@@ -72,7 +72,9 @@ class TemperatureException(Exception):
 class Thermostat:
     """Representation of a EQ3 Bluetooth Smart thermostat."""
 
-    def __init__(self, _mac, connection_cls=BTLEConnection, keep_connection=False):
+    def __init__(
+        self, _mac, connection_cls=BTLEConnection, keep_connection=False, max_retries=5
+    ):
         """Initialize the thermostat."""
 
         self._target_temperature = Mode.Unknown
@@ -95,9 +97,12 @@ class Thermostat:
         self._firmware_version = None
         self._device_serial = None
 
-        self._conn = connection_cls(_mac, keep_connection)
+        self._conn = connection_cls(_mac, keep_connection, max_retries)
         self._conn.set_callback(PROP_NTFY_HANDLE, self.handle_notification)
 
+    def set_max_retries(self, max_retries):
+        self._conn.set_max_retries(max_retries)
+
     def set_keep_connected(self, new_state):
         self._conn.set_keep_connected(new_state)
 
@@ -239,7 +244,7 @@ class Thermostat:
         return self._schedule
 
     def set_schedule(self, data):
-        """Sets the schedule for the given day. """
+        """Sets the schedule for the given day."""
         value = Schedule.build(data)
         self._conn.make_request(PROP_WRITE_HANDLE, value)
 
@@ -378,14 +383,14 @@ class Thermostat:
             "Window open config, temperature: %s duration: %s", temperature, duration
         )
         self._verify_temperature(temperature)
-        if duration.seconds < 0 and duration.seconds > 3600:
+        if duration < 0 and duration > 3600:
             raise ValueError
 
         value = struct.pack(
             "BBB",
             PROP_WINDOW_OPEN_CONFIG,
             int(temperature * 2),
-            int(duration.seconds / 300),
+            int(duration / 300),
         )
         self._conn.make_request(PROP_WRITE_HANDLE, value)
 
