@@ -3,9 +3,8 @@ import functools
 import struct
 import time
 
-from .. import fhem, utils
-from ..core.ble import BTLEConnection
-from ..generic import FhemModule
+from .. import fhem, generic, utils
+from ..core import ble
 
 DEFAULT_TIMEOUT = 1
 
@@ -42,9 +41,11 @@ HANDLE_RW_TIMERS = [
 ]
 
 
-class gfprobt(FhemModule):
+class gfprobt(generic.FhemModule):
     def __init__(self, logger):
         super().__init__(logger)
+        self._ble_lock = asyncio.Lock()
+        self._conn = None
         set_conf = {
             "update": {"help": "Retrieve values from GF Pro BT"},
             "off": {"help": "Turn off watering"},
@@ -55,13 +56,18 @@ class gfprobt(FhemModule):
                     "onseconds": {"default": 0, "format": "int", "optional": True}
                 },
                 "help": (
-                    "Turn on watering. You can use seconds as parameter to define the duration.<br>"
+                    "Turn on watering. You can use seconds as parameter to set the duration.<br>"
                     "Duration will be set directly on the device to have a safe switch off even if connectivity is broken"
                 ),
             },
             "devicename": {"help": "Set the devicename of the irrigation control"},
             "adjust": {
-                "help": "Adjust the timings with parameters:<br>percent<br>duration in minutes"
+                "args": ["percentage", "duration"],
+                "params": {
+                    "percentage": {"default": 0, "format": "int"},
+                    "duration": {"default": 360, "format": " int", "optional": True},
+                },
+                "help": "Adjust the timings with parameters:<br>percentage<br>duration in hours",
             },
             "eco": {"help": "Activate/decactivate eco mode"},
         }
@@ -72,32 +78,54 @@ class gfprobt(FhemModule):
     async def Define(self, hash, args, argsh):
         await super().Define(hash, args, argsh)
         if len(args) < 4:
-            return "Usage: define irrigation_control PythonModule gfprobt <MAC>"
+            return "Usage: define irrigation_control fhempy gfprobt <MAC>"
         self._mac = args[3]
         self.hash["MAC"] = self._mac
-        self._conn = BTLEConnection(
+        self._conn = ble.BTLEConnection(
             self._mac,
             keep_connected=True,
             connection_established_callback=self.write_password,
         )
-        self.create_async_task(self.update())
+        self.create_async_task(self.update_loop())
+
+    async def Undefine(self, hash):
+        if self._conn:
+            self._conn.set_keep_connected(False)
+        return await super().Undefine(self.hash)
 
     # write_password is running in another thread
     def write_password(self, mac):
         self._conn.write_characteristic(HANDLE_RW_PASSWORD, b"123456")
 
     async def set_update(self, hash, params):
-        self.create_async_task(self.update())
+        self.create_async_task(self.update_once())
 
     async def set_on(self, hash, params):
         onseconds = params["onseconds"]
-        utils.run_blocking_task(functools.partial(self.blocking_on, onseconds))
+        self.create_async_task(
+            self._blocking_set(functools.partial(self.blocking_on, onseconds))
+        )
 
     async def set_off(self, hash, params):
-        utils.run_blocking_task(functools.partial(self.blocking_off))
+        self.create_async_task(self._blocking_set(functools.partial(self.blocking_off)))
 
     async def set_toggle(self, hash, params):
-        utils.run_blocking_task(functools.partial(self.blocking_toggle))
+        self.create_async_task(
+            self._blocking_set(functools.partial(self.blocking_toggle))
+        )
+
+    async def set_adjust(self, hash, params):
+        self.create_async_task(
+            self._blocking_set(
+                functools.partial(
+                    self.blocking_adjust, params["percentage"], params["duration"]
+                )
+            )
+        )
+
+    async def _blocking_set(self, fct):
+        async with self._ble_lock:
+            utils.run_blocking(fct)
 
     def blocking_on(self, onseconds):
         self.blocking_update_watering()
@@ -113,11 +141,42 @@ class gfprobt(FhemModule):
         self._conn.write_characteristic(HANDLE_W_WATERING, b"\x00")
         self._conn.write_characteristic(HANDLE_W_WATERING, b"\x01")
 
-    async def update(self):
-        await utils.run_blocking(functools.partial(self.blocking_update))
+    def blocking_adjust(self, percentage, duration):
+        duration_sec = duration * 60 * 60
+
+        duration_hex = duration_sec.to_bytes(6, "little")
+        percentage_hex = percentage.to_bytes(2, "little")
+        self._conn.writeCharacteristic(
+            HANDLE_RW_INCREASEREDUCE, duration_hex + percentage_hex
+        )
+
+        self.write_offset()
+        self.commit_code()
+
+    async def update_loop(self):
+        while True:
+            try:
+                await self.update_once()
+            except asyncio.CancelledError:
+                break
+            except Exception:
+                self.logger.exception("Failed to update readings")
+            await asyncio.sleep(60)
+
+    async def update_once(self):
+        async with self._ble_lock:
+            await utils.run_blocking(functools.partial(self.blocking_update))
         await fhem.readingsBeginUpdate(self.hash)
-        await fhem.readingsBulkUpdateIfChanged(self.hash, "state", self._watering)
+        await fhem.readingsBulkUpdateIfChanged(
+            self.hash, "state", "on" if self._watering == 1 else "off"
+        )
         await fhem.readingsBulkUpdateIfChanged(self.hash, "watering", self._watering)
+        await fhem.readingsBulkUpdateIfChanged(
+            self.hash, "adjust_hours", self._adjust_hours
+        )
+        await fhem.readingsBulkUpdateIfChanged(
+            self.hash, "adjust_percentage", self._adjust_perc
+        )
         await fhem.readingsBulkUpdateIfChanged(
             self.hash, "batteryVoltage", self._battery
         )
@@ -158,13 +217,15 @@ class gfprobt(FhemModule):
         self._devname = self._conn.read_characteristic(HANDLE_RW_DEVNAME).decode(
             "utf-8"
         )
-        self._eco = str(self._conn.read_characteristic(HANDLE_RW_ECO_PART1))
-        self._eco += " " + str(self._conn.read_characteristic(HANDLE_RW_ECO_PART2))
+        self._eco = self._conn.read_characteristic(HANDLE_RW_ECO_PART1)
+        self._eco += self._conn.read_characteristic(HANDLE_RW_ECO_PART2)
         self._timeoffset = struct.unpack(
             "<I", self._conn.read_characteristic(HANDLE_RW_TIME_OFFSET)
         )[0]
         self._devmac = str(self._conn.read_characteristic(HANDLE_R_MAC))
         self._increasereduce = self._conn.read_characteristic(HANDLE_RW_INCREASEREDUCE)
+        self._adjust_hours = struct.unpack("<I", self._increasereduce[0:4])[0] / 3600
+        self._adjust_perc = struct.unpack("<h", self._increasereduce[4:])[0]
         self._raw_timers = {}
         for handle_timer in HANDLE_RW_TIMERS:
             self._raw_timers[handle_timer] = self._conn.read_characteristic(
@@ -177,9 +238,9 @@ class gfprobt(FhemModule):
         sec_since_mon = (
             now.tm_wday * 3600 * 24 + now.tm_hour * 3600 + now.tm_min * 60 + now.tm_sec
         )
-        sec_since_mon = struct.pack("<I", sec_since_mon)
+        sec_since_mon = sec_since_mon.to_byte(4, "little")
         self._conn.writeCharacteristic(HANDLE_RW_TIME_OFFSET, sec_since_mon)
 
     def commit_code(self):
-        self._conn.writeCharacteristic(HANDLE_W_COMMITCODE, b"00")
-        self._conn.writeCharacteristic(HANDLE_W_COMMITCODE, b"01")
+        self._conn.writeCharacteristic(HANDLE_W_COMMITCODE, b"\x00")
+        self._conn.writeCharacteristic(HANDLE_W_COMMITCODE, b"\x01")
