@@ -7,16 +7,16 @@ import typing
 
 from miio.click_common import DeviceGroupMeta
 
-from .. import fhem, utils
-from ..generic import FhemModule
+from .. import fhem, generic, utils
 
 
-class miio(FhemModule):
+class miio(generic.FhemModule):
     def __init__(self, logger):
         super().__init__(logger)
         self._set_list = {}
         self._device = None
         self._fct_update_tasks = {}
+        self._cmd_lock = asyncio.Lock()
         self._attr_update_functions = ""
         self._attr_list = {
             "update_functions": {
@@ -35,13 +35,13 @@ class miio(FhemModule):
         await super().Define(hash, args, argsh)
         self.hash = hash
         if len(args) < 6:
-            return "Usage: define miiodev PythonModule miio <TYPE> <IP> <TOKEN>"
+            return "Usage: define miiodev fhempy miio <TYPE> <IP> <TOKEN>"
         self._miio_devtype = args[3]
         self._miio_ip = args[4]
         self._miio_token = args[5]
 
         self._miio_device_class = None
-        for device_class in DeviceGroupMeta.device_classes:
+        for device_class in DeviceGroupMeta._device_classes:
             if device_class.get_device_group().name == self._miio_devtype:
                 self._miio_device_class = device_class
                 break
@@ -50,11 +50,11 @@ class miio(FhemModule):
 
         for dev_cmd in self._miio_device_class.get_device_group().commands.keys():
             self._set_list[dev_cmd] = {"function": "set_command", "default": None}
-            fct = getattr(self._miio_device_class, dev_cmd)
+            fct = self._miio_device_class._device_group_commands[dev_cmd].func
             sig = inspect.signature(fct)
+            self._set_list[dev_cmd]["help"] = []
             if len(list(sig.parameters)) > 1:
                 self._set_list[dev_cmd]["args"] = []
-                self._set_list[dev_cmd]["help"] = []
                 for par in sig.parameters:
                     if sig.parameters[par].name == "self":
                         continue
@@ -71,13 +71,26 @@ class miio(FhemModule):
                             )
                         elif inspect.isclass(annot) and issubclass(annot, bool):
                             self._set_list[dev_cmd]["options"] = "on,off"
-                self._set_list[dev_cmd]["help"] = "Arguments: " + " ".join(
-                    self._set_list[dev_cmd]["help"]
-                )
+
+            helptext = ""
+            if (
+                "help" in self._miio_device_class._device_group_commands[dev_cmd].kwargs
+                and self._miio_device_class._device_group_commands[dev_cmd].kwargs[
+                    "help"
+                ]
+                is not None
+            ):
+                helptext = self._miio_device_class._device_group_commands[
+                    dev_cmd
+                ].kwargs["help"]
+            self._set_list[dev_cmd]["help"] = (
+                helptext + "<br>Arguments: " + " ".join(self._set_list[dev_cmd]["help"])
+            )
 
         self.set_set_config(self._set_list)
         self._device = self._miio_device_class(ip=self._miio_ip, token=self._miio_token)
         await fhem.readingsSingleUpdateIfChanged(hash, "state", "active", 1)
+        await self.set_attr_update_functions(self.hash)
 
     async def set_attr_update_functions(self, hash):
         for task in self._fct_update_tasks.copy():
@@ -96,12 +109,15 @@ class miio(FhemModule):
     async def fct_update_loop(self, fct_name, sec):
         while True:
             try:
-                await self.send_command(fct_name, None)
+                async with self._cmd_lock:
+                    await asyncio.sleep(1)
+                    await self.send_command(fct_name, None, raise_exc=True)
                 await fhem.readingsSingleUpdateIfChanged(
                     self.hash, "presence", "online", 1
                 )
             except Exception:
-                self.logger.error(f"Failed to send_command: {fct_name}")
+                if fct_name != "status":
+                    self.logger.error(f"Failed to send_command: {fct_name}")
                 await fhem.readingsSingleUpdateIfChanged(
                     self.hash, "presence", "offline", 1
                 )
@@ -118,12 +134,14 @@ class miio(FhemModule):
         except ValueError:
             return False
 
-    async def send_command(self, fct_name, params):
-        fct = getattr(self._device, fct_name)
+    async def send_command(self, fct_name, params, raise_exc=False):
+        fct = self._device._device_group_commands[fct_name].func
         sig = inspect.signature(fct)
         args = []
         for par_name in sig.parameters:
-            if params[par_name] is None:
+            if par_name == "self":
+                continue
+            if par_name not in params or params[par_name] is None:
                 continue
             ann = sig.parameters[par_name].annotation
             if inspect.isclass(ann) and issubclass(ann, enum.Enum):
@@ -132,6 +150,8 @@ class miio(FhemModule):
                 args.append(params[par_name] == "on")
             elif ann == typing.List:
                 args.append(json.loads(params[par_name]))
+            elif hasattr(ann, "_name") and ann._name == "Tuple":
+                args.append(tuple(map(int, params[par_name].split(","))))
             elif ann == inspect.Signature.empty:
                 if self.is_number(params[par_name]):
                     args.append(float(params[par_name]))
@@ -140,7 +160,15 @@ class miio(FhemModule):
             else:
                 args.append(ann(params[par_name]))
         # call function with arguments
-        reply = await utils.run_blocking(functools.partial(fct, *args))
+        call_fct = getattr(self._device, fct_name)
+        try:
+            reply = await utils.run_blocking(functools.partial(call_fct, *args))
+        except Exception as exc:
+            if raise_exc:
+                raise exc
+            self.logger.exception(f"Failed to call {fct_name}")
+            return
+
         # handle reply
         if hasattr(reply, "__dict__"):
             await fhem.readingsBeginUpdate(self.hash)
@@ -155,15 +183,15 @@ class miio(FhemModule):
                         continue
                     try:
                         for data_name in st[prop]:
-                            await fhem.readingsBulkUpdateIfChanged(
-                                self.hash, prop + "_" + data_name, st[prop][data_name]
+                            await fhem.readingsBulkUpdate(
+                                self.hash,
+                                prop + "_" + data_name,
+                                st[prop][data_name],
                             )
                     except Exception:
-                        await fhem.readingsBulkUpdateIfChanged(
-                            self.hash, prop, st[prop]
-                        )
+                        await fhem.readingsBulkUpdate(self.hash, prop, st[prop])
             except Exception:
-                pass
+                self.logger.exception("Failed to update readings")
             await fhem.readingsEndUpdate(self.hash, 1)
         else:
             if (
@@ -171,9 +199,7 @@ class miio(FhemModule):
                 and isinstance(reply, str)
                 and reply.lower() != "['ok']"
             ):
-                await fhem.readingsSingleUpdateIfChanged(
-                    self.hash, fct.__name__, reply, 1
-                )
+                await fhem.readingsSingleUpdate(self.hash, fct.__name__, reply, 1)
         if fct_name != "status":
             # wait before status request as it takes some time for the miio device
             # to change the state
