@@ -1,16 +1,13 @@
 import asyncio
 import concurrent
 import functools
-import glob
 import os
 import time
-from threading import Thread
 
 import cv2
 import numpy as np
-import tflit
 from fhempy.lib.generic import FhemModule
-from tflite_runtime.interpreter import Interpreter, load_delegate
+from tflite_runtime.interpreter import Interpreter
 
 from .. import fhem, utils
 
@@ -41,7 +38,7 @@ class object_detection(FhemModule):
     async def Define(self, hash, args, argsh):
         await super().Define(hash, args, argsh)
         if len(args) < 5:
-            return "Usage: define obj_detection PythonModule object_detection <image/stream> <LOCATION>"
+            return "Usage: define obj_detection fhempy object_detection <image/stream> <LOCATION>"
         if args[3] == "image":
             if args[4][0] == "/":
                 self._source_uri = args[4]
