@@ -24,21 +24,22 @@ async def test_update(mocker):
 
     mock_fhem.mock_module(mocker)
     websocket = MagicMock()
-    fhempy = fhem_pythonbinding.PyBinding(websocket)
+    fhempy = fhem_pythonbinding.fhempy(websocket)
 
     update_msg = {
         "id": "123",
         "msgtype": "update",
         "NAME": "testdevice",
+        "PYTHONTYPE": "testname",
+        "FHEMPYTYPE": "testname",
         "args": [],
         "argsh": {},
         "defargs": [],
         "defargsh": {},
     }
     update_text = json.dumps(update_msg)
-    os._exit = sys.exit
-    with pytest.raises(SystemExit) as pytest_wrapped_e:
-        await fhempy.onMessage(update_text)
-    assert pytest_wrapped_e.type == SystemExit
-    assert pytest_wrapped_e.value.code == 1
+    await fhempy.onMessage(update_text)
+
+    assert fhem_pythonbinding.exit_code == 1
+    assert fhem_pythonbinding.stop_event.is_set() is True
     assert is_installed("fhempy")
