@@ -40,6 +40,7 @@ def mock_module(mocker):
     mocker.patch("fhempy.lib.fhem.convertValue", convertValue)
     mocker.patch("fhempy.lib.fhem.send_version", send_version)
     mocker.patch("fhempy.lib.fhem.setFunctionInactive", do_nothing)
+    mocker.patch("fhempy.lib.fhem.init_done", init_done)
 
 
 def do_nothing(param):
@@ -62,6 +63,10 @@ def setFunctionInactive(hash):
         )
 
 
+async def init_done(hash):
+    return 1
+
+
 async def getUniqueId(hash):
     return "12345678901234567890123456789012"
 
